pub struct SweepLineRangeBuffers { /* private fields */ }Expand description
Holds some buffers that are used when iterating over a sweep-line.
Save on re-allocation by allocating this once and reusing it in multiple calls to
SweepLine::next_range.